#!/usr/bin/perl -w
#
# Perl based cgi-bin script for retrieving information from Nagios
# in Grid monitoring data exchange format 
# (see https://twiki.cern.ch/twiki/bin/view/LCG/GridMonitoringDataExchangeStandard)
# Copyright (c) 2007 Emir Imamagic
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

use strict;
use CGI qw(:standard);
use CGI::Carp qw(warningsToBrowser fatalsToBrowser);
use Nagios;
use XML::Writer;
use Date::Format;

sub get_sitename_for_wlcg();

my $host = param('Host_name');

exit 3 if ($host && !is_hostname($host));

my $serviceMetric = "";
my $sitename = get_sitename_for_wlcg();
exit 3 if !$sitename;

my $VONAME = 'dteam';

my %ERRORS = (
    0 => 'ok',
    1 => 'warning',
    2 => 'critical',
    3 => 'unknown',
);

# Get sitename from /etc/sysconfig/lcg
sub get_sitename_for_wlcg() {
        my $SYSCONFIG="/etc/sysconfig/lcg";
        my $site;
        open(SYSCONFIG, "<$SYSCONFIG") || return undef;
        while(<SYSCONFIG>) {
                next if /^\s*#/;
                if(/^\s*export\s*SITE_NAME\s*=\s*(\w*)$/) {
                        $site = $1;
                }
        }
        return $site;
}

# taken from Nagios utils.pm
sub is_hostname {
    my $host1 = shift;
    return 0 unless defined $host1;
    if ($host1 =~ m/^[\d\.]+$/ && $host1 !~ /\.$/) {
        if ($host1 =~ m/^[0-9]+\.[0-9]+\.[0-9]+\.[0-9]+$/) {
            return 1;
        } else {
            return 0;
        }
    } elsif ($host1 =~ m/^[a-zA-Z0-9][-a-zA-Z0-9]+(\.[a-zA-Z0-9][-a-zA-Z0-9]+)*\.?$/) {
        return 1;
    } else {
        return 0;
    }
}


sub response {
    my $site = shift;
    my $host = shift || "";
    my $serviceMetric = shift || "";
    
    my $nagios = Nagios->new();
    my $confServiceAttrs = ["servicegroups"];
    my $statusHostAttrs = ["current_state","plugin_output","last_check"];
    my $statusServiceAttrs = ["current_state","plugin_output","last_check","check_command"];
    
    $nagios->fetchStatusHostData($statusHostAttrs);
#    $nagios->fetchConfHostData($confServiceAttrs);
    $nagios->fetchStatusServiceData($statusServiceAttrs);
    $nagios->fetchConfServiceData($confServiceAttrs);
    
    my $nagiosFilter = {};
    if ($host) {
        $nagiosFilter->{host_name} = $host;
        $nagios->filterHosts($nagiosFilter);
    }
    
    my $xmlResponse;
    my $writer = XML::Writer->new(OUTPUT => \$xmlResponse);
    
    $writer->startTag("root", xmlns => "http://cern.ch/grid-mon/2007/05/mon-exchange-schema/");
    $writer->startTag("site", name => $site);
    
    foreach my $nHost (keys %{$nagios->{HOSTS}}) {
        my $nodeType = "";
        my $currService = "";
        
        $writer->startTag("host", name => $nagios->{HOSTS}->{$nHost}->{host_name});
         
        # add hostMetric host-status
        $writer->startTag("hostMetric", name => 'org.wlcg.host-status');
        $writer->startTag("measurement");
        $writer->startTag("status");
        $writer->characters($ERRORS{$nagios->{HOSTS}->{$nHost}->{current_state}});
        $writer->endTag("status");
        $writer->startTag("summary");
        $writer->characters($nagios->{HOSTS}->{$nHost}->{plugin_output});
        $writer->endTag("summary");
        $writer->startTag("timestamp");
        # 2007-02-09T14:42:20Z
        $writer->characters(time2str("%Y-%m-%dT%H:%M:%SZ",$nagios->{HOSTS}->{$nHost}->{last_check},"UTC"));
        $writer->endTag("timestamp");
        $writer->endTag("measurement");
        $writer->endTag("hostMetric");
        
        my $hostServices = $nagios->{HOSTS}->{$nHost}->{services};
        foreach my $service (sort keys %{$hostServices}) {
            if ($hostServices->{$service}->{service_description} =~ /^org.wlcg./) {
                my $serviceType;
                
                foreach my $nodeT (split(/,/, $hostServices->{$service}->{servicegroups})) {
                    print $nodeT;
                    if ($nodeT ne 'local') {
                        $serviceType = $nodeT;
                        last;
                    }
                }
            
                if (!$currService || $currService ne $serviceType) {
                    my $serviceURI;    
                
                    if ($currService) {
                        $writer->endTag("metricGroup");
                        $writer->endTag("service");
                    }
                    $currService = $serviceType;
                    
                    if ($hostServices->{$service}->{check_command} =~ /^check_wlcg!\S+?!(\S+?)!.*/) {
                        $serviceURI = $1;
                    } else {
                        $serviceURI = $nagios->{HOSTS}->{$nHost}->{host_name};
                    }
                    
                    foreach my $nodeT (split(/,/, $hostServices->{$service}->{servicegroups})) {
                        if (chop($nodeT) ne 'local') {
                            $serviceType = chop($nodeT);
                            last;
                        }
                    }
                    
                    $writer->startTag("service", endpoint => $serviceURI, type => $currService);
                    $writer->startTag("isMonitored");
                    $writer->characters('true');
                    $writer->endTag("isMonitored");
                    
                    $writer->startTag("metricGroup", groupBy => 'calculatedForVO', value=> $VONAME);
                }
                
                $writer->startTag("serviceMetric", name => $hostServices->{$service}->{service_description});
                $writer->startTag("measurement");
        
                $writer->startTag("status");
                $writer->characters($ERRORS{$hostServices->{$service}->{current_state}});
                $writer->endTag("status");
        
                $writer->startTag("summary");
                $writer->characters($hostServices->{$service}->{plugin_output});
                $writer->endTag("summary");
        
                $writer->startTag("timestamp");
                # 2007-02-09T14:42:20Z
                $writer->characters(time2str("%Y-%m-%dT%H:%M:%SZ",$hostServices->{$service}->{last_check},"UTC"));
                $writer->endTag("timestamp");
        
                $writer->endTag("measurement");
                $writer->endTag("serviceMetric");
            }
        }
        if ($currService) {
            $writer->endTag("metricGroup");
            $writer->endTag("service");
        }
        $writer->endTag("host");
    }
    $writer->endTag("site");
    $writer->endTag("root");
    
    my ($xml_string) = @_;
    my $length = length($xmlResponse);
        print <<"EOD";
Status: 200 OK
Content-type: text/xml
Content-length: $length

EOD
    print $xmlResponse;
}

my $result = response $sitename, $host, $serviceMetric;


